using System;
using System.Collections.Generic;
using System.IO;
using System.Threading;
using Cysharp.Threading.Tasks;
using UnityEngine;
using UnityEngine.Networking;

namespace MotionFramework.Scripts.Runtime.Engine.Engine.Network.WebRequest
{
    /// <summary>
    ///  WebRequest 
    /// </summary>
    public sealed class WebRequestManager : ModuleSingleton<WebRequestManager>, IModule
    {
        /// <summary>
        /// Getȡ
        /// </summary>-
        /// <param name="url"></param>
        /// <param name="onProgress"></param>
        /// <param name="cancellationToken"></param>
        /// <returns></returns>
        public async UniTask<string> GetTextAsync(string url, Action<float> onProgress = null, CancellationToken cancellationToken = default)
        {
            using (UnityWebRequest request = UnityWebRequest.Get(url))
            {
                await SendRequestWithProgress(request, onProgress, cancellationToken);
                if (request.result == UnityWebRequest.Result.ConnectionError || request.result == UnityWebRequest.Result.ProtocolError)
                {
                    Debug.LogError($"쳣 {request.error}");
                    return null;
                }
                return request.downloadHandler.text;
            }
        }

        /// <summary>
        /// ȡͼƬ
        /// </summary>
        /// <param name="url"></param>
        /// <param name="onProgress"></param>
        /// <param name="cancellationToken"></param>
        /// <returns></returns>
        public async UniTask<Texture2D> GetImageAsync(string url, Action<float> onProgress = null, CancellationToken cancellationToken = default)
        {
            using (UnityWebRequest request = UnityWebRequestTexture.GetTexture(url))
            {
                await SendRequestWithProgress(request, onProgress, cancellationToken);
                if (request.result == UnityWebRequest.Result.ConnectionError || request.result == UnityWebRequest.Result.ProtocolError)
                {
                    Debug.LogError($"쳣 {request.error}");
                    return null;
                }
                return DownloadHandlerTexture.GetContent(request);
            }
        }

        /// <summary>
        /// ȡƵ
        /// </summary>
        /// <param name="url"></param>
        /// <param name="localPath"></param>
        /// <param name="onProgress"></param>
        /// <param name="cancellationToken"></param>
        /// <returns></returns>
        public async UniTask<string> GetVideoAsync(string url, string localPath, Action<float> onProgress = null, CancellationToken cancellationToken = default)
        {
            using (UnityWebRequest request = UnityWebRequest.Get(url))
            {
                await SendRequestWithProgress(request, onProgress, cancellationToken);
                if (request.result == UnityWebRequest.Result.ConnectionError || request.result == UnityWebRequest.Result.ProtocolError)
                {
                    Debug.LogError($"쳣 {request.error}");
                    return null;
                }
                File.WriteAllBytes(localPath, request.downloadHandler.data);
                return localPath;
            }
        }

        /// <summary>
        /// ȡļ
        /// </summary>
        /// <param name="url"></param>
        /// <param name="onProgress"></param>
        /// <param name="cancellationToken"></param>
        /// <returns></returns>
        public async UniTask<byte[]> GetFileAsync(string url, Action<float> onProgress = null, CancellationToken cancellationToken = default)
        {
            using (UnityWebRequest request = UnityWebRequest.Get(url))
            {
                await SendRequestWithProgress(request, onProgress, cancellationToken);
                if (request.result == UnityWebRequest.Result.ConnectionError || request.result == UnityWebRequest.Result.ProtocolError)
                {
                    Debug.LogError($"쳣 {request.error}");
                    return null;
                }
                return request.downloadHandler.data;
            }
        }

        /// <summary>
        /// ȡ
        /// </summary>
        /// <param name="url"></param>
        /// <param name="jsonData"></param>
        /// <param name="onProgress"></param>
        /// <param name="cancellationToken"></param>
        /// <returns></returns>
        public async UniTask<string> PostTextAsync(string url, string jsonData, Action<float> onProgress = null, CancellationToken cancellationToken = default)
        {
            using (UnityWebRequest request = new UnityWebRequest(url, UnityWebRequest.kHttpVerbPOST))
            {
                byte[] bodyRaw = new System.Text.UTF8Encoding().GetBytes(jsonData);
                request.uploadHandler = new UploadHandlerRaw(bodyRaw);
                request.downloadHandler = new DownloadHandlerBuffer();
                request.SetRequestHeader("Content-Type", "application/json");

                await SendRequestWithProgress(request, onProgress, cancellationToken);
                if (request.result == UnityWebRequest.Result.ConnectionError || request.result == UnityWebRequest.Result.ProtocolError)
                {
                    Debug.LogError($"쳣 {request.error}");
                    return null;
                }
                return request.downloadHandler.text;
            }
        }

        /// <summary>
        /// ȡͼƬ
        /// </summary>
        /// <param name="url"></param>
        /// <param name="jsonData"></param>
        /// <param name="onProgress"></param>
        /// <param name="cancellationToken"></param>
        /// <returns></returns>
        public async UniTask<Texture2D> PostImageAsync(string url, string jsonData, Action<float> onProgress = null, CancellationToken cancellationToken = default)
        {
            using (UnityWebRequest request = new UnityWebRequest(url, UnityWebRequest.kHttpVerbPOST))
            {
                byte[] bodyRaw = new System.Text.UTF8Encoding().GetBytes(jsonData);
                request.uploadHandler = new UploadHandlerRaw(bodyRaw);
                request.downloadHandler = new DownloadHandlerTexture();
                request.SetRequestHeader("Content-Type", "application/json");

                await SendRequestWithProgress(request, onProgress, cancellationToken);
                if (request.result == UnityWebRequest.Result.ConnectionError || request.result == UnityWebRequest.Result.ProtocolError)
                {
                    Debug.LogError($"쳣 {request.error}");
                    return null;
                }
                return DownloadHandlerTexture.GetContent(request);
            }
        }

        /// <summary>
        /// ȡƵ
        /// </summary>
        /// <param name="url"></param>
        /// <param name="jsonData"></param>
        /// <param name="localPath"></param>
        /// <param name="onProgress"></param>
        /// <param name="cancellationToken"></param>
        /// <returns></returns>
        public async UniTask<string> PostVideoAsync(string url, string jsonData, string localPath, Action<float> onProgress = null, CancellationToken cancellationToken = default)
        {
            using (UnityWebRequest request = new UnityWebRequest(url, UnityWebRequest.kHttpVerbPOST))
            {
                byte[] bodyRaw = new System.Text.UTF8Encoding().GetBytes(jsonData);
                request.uploadHandler = new UploadHandlerRaw(bodyRaw);
                request.downloadHandler = new DownloadHandlerBuffer();
                request.SetRequestHeader("Content-Type", "application/json");

                await SendRequestWithProgress(request, onProgress, cancellationToken);
                if (request.result == UnityWebRequest.Result.ConnectionError || request.result == UnityWebRequest.Result.ProtocolError)
                {
                    Debug.LogError($"쳣 {request.error}");
                    return null;
                }
                File.WriteAllBytes(localPath, request.downloadHandler.data);
                return localPath;
            }
        }

        /// <summary>
        /// ȡļ
        /// </summary>
        /// <param name="url"></param>
        /// <param name="jsonData"></param>
        /// <param name="onProgress"></param>
        /// <param name="cancellationToken"></param>
        /// <returns></returns>
        public async UniTask<byte[]> PostFileAsync(string url, string jsonData, Action<float> onProgress = null, CancellationToken cancellationToken = default)
        {
            using (UnityWebRequest request = new UnityWebRequest(url, UnityWebRequest.kHttpVerbPOST))
            {
                byte[] bodyRaw = new System.Text.UTF8Encoding().GetBytes(jsonData);
                request.uploadHandler = new UploadHandlerRaw(bodyRaw);
                request.downloadHandler = new DownloadHandlerBuffer();
                request.SetRequestHeader("Content-Type", "application/json");

                await SendRequestWithProgress(request, onProgress, cancellationToken);
                if (request.result == UnityWebRequest.Result.ConnectionError || request.result == UnityWebRequest.Result.ProtocolError)
                {
                    Debug.LogError($"쳣 {request.error}");
                    return null;
                }
                return request.downloadHandler.data;
            }
        }

        /// <summary>
        /// ύ
        /// </summary>
        /// <param name="url"></param>
        /// <param name="formFields"></param>
        /// <param name="onProgress"></param>
        /// <param name="cancellationToken"></param>
        /// <returns></returns>
        public async UniTask<string> PostFormAsync(string url, Dictionary<string, string> formFields, Action<float> onProgress = null, CancellationToken cancellationToken = default)
        {
            using (UnityWebRequest request = UnityWebRequest.Post(url, formFields))
            {
                await SendRequestWithProgress(request, onProgress, cancellationToken);
                if (request.result == UnityWebRequest.Result.ConnectionError || request.result == UnityWebRequest.Result.ProtocolError)
                {
                    Debug.LogError($"쳣 {request.error}");
                    return null;
                }
                return request.downloadHandler.text;
            }
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="request"></param>
        /// <param name="onProgress"></param>
        /// <param name="cancellationToken"></param>
        /// <returns></returns>
        private async UniTask SendRequestWithProgress(UnityWebRequest request, Action<float> onProgress, CancellationToken cancellationToken)
        {
            // ȷʹawait첽
            var operation = request.SendWebRequest();

            // ѭǷɣ½
            while (!operation.isDone)
            {
                onProgress?.Invoke(request.downloadProgress);

                //Debug.Log($"Progress: {request.downloadProgress * 100:F2}%");
                await UniTask.Delay(100, cancellationToken: cancellationToken);  // ÿ100һν
            }
            onProgress?.Invoke(1.0f);
            // һνȸ£ȷʾΪ100%
            Debug.Log("Progress: 100%");
        }

        public void OnCreate(object createParam)
        {
        }

        public void OnUpdate()
        {
        }

        public void OnDestroy()
        {
        }

        public void OnGUI()
        {
        }
    }
}
